<?php

namespace Picea\Caching;

use Picea\Compiler\Context;

class Opcache implements Cache {

    protected string $cacheFolder = "picea";

    protected string $cachePath = "";

    protected array $compiled = [];

    public function __construct(?string $cachePath = null)
    {
        $this->cachePath = $cachePath ?? sys_get_temp_dir();
    }

    public function load(string $viewPath, ...$arguments) : object
    {
        if ( empty($viewPath) ) {
            throw new \RuntimeException("Given view path is empty.");
        }

        if ( ! $this->compiled($viewPath) ) {
            throw new \RuntimeException("You must compile your view `$viewPath` before trying to load it.");
        }

        $compiledContext = $this->compiled[$viewPath];

        if ( $compiledContext['extends'] ) {
            $this->load($compiledContext['extends'], ...$arguments);
        }

        $fullName = isset($compiledContext['namespace']) ? "\\{$compiledContext['namespace']}\\{$compiledContext['classname']}" : $compiledContext['classname'];

        return new $fullName(...$arguments);
    }

    public function save(Context $context) : bool
    {
        $fileName = $context->compiledClassPath();
        $this->validateCachePath();
        file_put_contents($this->cachePath($context->viewPath), $context->compiledSource);

        return false;
    }

    public function compiled(string $viewPath) : bool
    {
        if ( false !== ( $this->compiled[$viewPath] ?? false ) ) {
            return true;
        }

        if ( file_exists($path = $this->cachePath($viewPath)) ) {
            $this->compiled[$viewPath] = include($path);
            return true;
        }

        return false;
    }

    protected function validateCachePath() : bool
    {
        $fullPath = $this->cachePath();

        if ( ! file_exists($fullPath) ) {
            mkdir($fullPath, 0750, true);
        }

        if ( ! is_writeable($fullPath) ) {
            throw new \RuntimeException(
                sprintf("Given cache path `%s` is not writeable by `%s`", $fullPath, static::class)
            );

            return false;
        }

        return true;
    }

    protected function cachePath(string $fileName = "") : string
    {
        return implode(DIRECTORY_SEPARATOR, array_filter([ $this->cachePath, $this->cacheFolder, $fileName ? str_replace([ "/", DIRECTORY_SEPARATOR ], "~", $fileName . ".php") : null ]));
    }
}
